/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/healthcheck/HealthCheckRepositoryContrib', [ 'Nexus/healthcheck/HealthCheckUtil' ], function() {

  HealthCheckRepositoryContrib = function(config) {
      var me = this;
      me.reset = function(config) {
          var config = config || {};
          var defaultConfig = {
              isHealthCheckUpdate : false,
              healthCheckWindow : null,
              healthCheckStatusTask : null,
              healthCheckDataMap : [],
              healthCheckCellMouse : false,
              rhcAnalyzingState : true,
              rhcToolbarHeight : 26,
              healthCheckStatusError : false,
              healthCheckTooltip : null,
              grid : null
          };
          Ext.apply(me, config, defaultConfig);

          if (me.grid) {
              me.grid.store.addListener('load', function(store, records, options) {
                  // force at least 1 request
                  me.rhcAnalyzingState = true;
                  me.startRHCPollTask();
                  me.generateHealthCheckTooltip();
              });

              me.grid.store.addListener('add', function(store, records, index) {
                  if (records.length == 1) {
                      if (!me.healthCheckDataMap[records[0].get('id')] && me.isProxyRepository(records[0])) {
                          me.healthCheckDataMap[records[0].get('id')] = {
                              status : 'disabled'
                          };
                      }
                  }
              });

              me.grid.mon(me.grid, 'mouseover', function(e, t) {
                  var row;
                  var row = me.grid.getView().findRowIndex(t);
                  var col = me.grid.getView().findCellIndex(t);
                  me.healthCheckCellMouse = false;
                  if (row !== false && col == me.grid.getColumnModel().getIndexById('healthCheckColumn')) {
                      var record = me.grid.store.getAt(row);
                      var healthCheckData = me.healthCheckDataMap[record.get('id')];
                      if (!me.healthCheckStatusError && healthCheckData && healthCheckData.status == 'enabled' && me.isRepositoryInService(record)) {
                          me.healthCheckCellMouse = true;
                          me.openSummary(Ext.get(me.grid.getView().getCell(row, col)), record);
                      }
                  }
              });

              me.grid.mon(me.grid, 'mouseout', function(e, t) {
                  me.healthCheckCellMouse = false;
              });
          }
      };

      me.reset(config);

      me.isRepositoryInService = function(record) {
          var status = record.get('status');
          if (status) {
              status = status.localStatus;
          }
          return status == 'IN_SERVICE';
      };

      me.isProxyRepository = function(record) {
          return record.get('repoType') == 'proxy';
      };

      me.isUnsupportedRepository = function(record) {
        return !me.isProxyRepository(record) || record.get('repoPolicy') == 'Snapshot' ||  !me.isRepositoryInService(record);
      };

      me.updateColumns = function(columns) {
          columns.splice(8, 0, {
              name : 'resourceURI',
              header : 'Health Check',
              id : 'healthCheckColumn',
              width : 100,
              renderer : function(value, metadata, record, rowIndex, colIndex, store) {
                  var healthCheckData = me.healthCheckDataMap[record.get('id')];

                  if (me.healthCheckStatusError) {
                      return '<div><img class="opaqueWarning" src="' + Sonatype.HealthCheckUtil.getHealthCheckResourceUrl('images/exclamation.png') + '"></div>';
                  } else if (me.isUnsupportedRepository(record)) {
                      return '';
                  } else if (!healthCheckData) {
                      return 'Loading...';
                  } else if (healthCheckData.status == 'enabled') {
                      // enabled, so add link to open iframe
                      metadata.css = 'cursorPointerCell badgeCell';
                      return '<div><img src="' + Sonatype.HealthCheckUtil.getHealthCheckResourceUrl('images/security-alert.png') + '">&nbsp;'
                              + healthCheckData.securityIssueCount + '&nbsp;&nbsp;<img src="' + Sonatype.HealthCheckUtil.getHealthCheckResourceUrl('images/license-alert.png')
                              + '" style="margin-left:10px">&nbsp;' + healthCheckData.licenseIssueCount + '</div>';
                  } else if (healthCheckData.status == 'analyzing') {
                      return '<div><img src="' + Sonatype.HealthCheckUtil.getHealthCheckResourceUrl('images/analyzing.png') + '"></div>';
                  } else {
                    return '';
                  }
              }
          });
      };

      me.openSummary = function(cellEl, record, defferred) {
          if (defferred) {
              if (me.healthCheckCellMouse) {
                  me.healthCheckSummaryClick(record, cellEl.getLeft() + 100, cellEl.getTop());
              }
          } else {
              me.openSummary.defer(500, me, [ cellEl, record, true ]);
          }
      };

      me.startRHCPollTask = function() {
          me.healthCheckStatusTask = {
              run : function() {
                  // if another task has started or if panel isn't around, return false to stop the task
                  if (!Ext.get(this.gridComponentId) || me.healthCheckStatusTask != this) {
                      return false;
                  }
                  // if nothing is 'analyzing' simply return and check next time (note this will not stop the task, only returning false will)
                  if (!me.rhcAnalyzingState) {
                      return;
                  }
                  Ext.Ajax.request({
                      callback : function(options, success, response) {
                          if (success) {
                              me.healthCheckStatusError = false;
                              var json = Ext.decode(response.responseText);
                              var store = me.grid.getStore();
                              me.rhcAnalyzingState = false;
                              for ( var i = 0; i < json.repositories.length; i++) {
                                  if (json.repositories[i].status == 'analyzing') {
                                      me.rhcAnalyzingState = true;
                                  }
                                  var record = store.getAt(store.findBy(function(record, id) {
                                      if (record.get('id') == json.repositories[i].id) {
                                          return true;
                                      }
                                      return false;
                                  }));

                                  if (record) {
                                      var data = me.healthCheckDataMap[record.get('id')];
                                      me.healthCheckDataMap[record.get('id')] = json.repositories[i];

                                      // TODO: really need a good js deep equals here...
                                      if (!data || data.healthCheckSummaryUrl != json.repositories[i].healthCheckSummaryUrl || data.iframeWidth != json.repositories[i].iframeWidth
                                              || data.iframeHeight != json.repositories[i].iframeHeight || data.status != json.repositories[i].status
                                              || data.securityIssueCount != json.repositories[i].securityIssueCount
                                              || data.licenseIssueCount != json.repositories[i].licenseIssueCount) {
                                          me.grid.getView().refreshRow(record);
                                      }
                                  }
                              }
                          } else {
                              if (!me.healthCheckStatusError) {
                                  me.healthCheckStatusError = true;
                                  me.grid.getView().refresh();
                              }
                              //don't want to continue to poll while in error state
                              me.rhcAnalyzingState = false;
                          }
                      },
                      url : Sonatype.HealthCheckUtil.getBase() + '/healthCheckStatus',
                      // hide the status when connection to host is lost
                      suppressStatus : [ 0, 500 ]
                  });
              },
              interval : 10000,
              gridComponentId : me.grid.getId()
          };

          Ext.TaskMgr.start(me.healthCheckStatusTask);
      };

      me.getHealthCheckSummarySizeParams = function(healthcheckdata) {
          // we are just assigning defaults here in case the cookie isn't available
          var sizeDeltas = Ext.state.Manager.get('healthcheck-summary-size-delta', {
              width : 0,
              height : 0
          });
          return {
              width : healthcheckdata.iframeWidth + sizeDeltas.width,
              height : healthcheckdata.iframeHeight + sizeDeltas.height
          };
      };

      me.healthCheckSummaryClick = function(record, x, y) {
          if (Sonatype.HealthCheckUtil.hasSummaryReadPerm()) {
              var bbarItems = [];

              var healthcheckdata = me.healthCheckDataMap[record.get('id')];

              var url = healthcheckdata.healthCheckSummaryUrl;

              if (!me.healthCheckWindow) {
                  Sonatype.Events.fireEvent('healthCheckWindowInit', me, bbarItems, record, healthcheckdata);

                  var sizeParams = me.getHealthCheckSummarySizeParams(healthcheckdata);

                  me.healthCheckWindow = new Ext.Window({
                      id : 'healthCheckWindow',
                      closable : false,
                      frame : false,
                      border : true,
                      layout : 'fit',
                      width : sizeParams.width,
                      height : sizeParams.height + (bbarItems.length > 0 ? me.rhcToolbarHeight : 0),
                      modal : false,
                      constrain : true,
                      resizable : true,
                      draggable : true,
                      x : x ? x : undefined,
                      y : y ? y : undefined,
                      bbar : bbarItems.length > 0 ? bbarItems : undefined,
                      items : [ {
                          xtype : 'iframepanel',
                          width : sizeParams.width,
                          height : sizeParams.height,
                          closable : true,
                          loadMask : {
                              msg : 'Loading Health Check Summary...'
                          },
                          autoShow : true,
                          id : 'healthCheckIframe',
                          defaultSrc : url,
                          disableMessaging : false
                      } ],
                      listeners : {
                          resize : {
                              fn : function(window, width, height) {
                                  Ext.state.Manager.set('healthcheck-summary-size-delta', {
                                      width : width - window.healthcheckdata.iframeWidth,
                                      height : height - (window.getBottomToolbar() ? me.rhcToolbarHeight : 0) - window.healthcheckdata.iframeHeight
                                  });
                              }
                          }
                      },
                      healthcheckdata : healthcheckdata
                  });
                  me.healthCheckWindow.show();
                  me.healthCheckWindow.el.on('mouseover', function() {
                      me.healthCheckWindow.windowMouse = true;
                      me.healthCheckWindow.forceClose = false;
                  });
                  me.healthCheckWindow.el.on('mouseout', function() {
                      var resizeElement = Ext.getDom('healthCheckWindow-rzproxy');
                      if (Ext.isEmpty(resizeElement) || resizeElement.style["display"] != 'block') {
                          me.healthCheckWindow.windowMouse = false;
                          me.healthCheckWindow.forceClose = true;
                      }
                  });

                  var closeFunction = function() {
                      if (me.healthCheckWindow) {
                          if (me.healthCheckWindow.forceClose || (!me.healthCheckWindow.windowMouse && !me.healthCheckCellMouse)) {
                              me.closeHealthCheckWindow();
                          } else {
                              this.defer(1000, this);
                          }
                      }
                  };
                  closeFunction.defer(1000, closeFunction);
              } else {
                  var iframeContainer = me.healthCheckWindow.getComponent('healthCheckIframe');
                  if (iframeContainer.defaultSrc != url) {
                      me.healthCheckWindow.healthcheckdata = healthcheckdata;
                      iframeContainer.defaultSrc = url;
                      iframeContainer.setSrc();
                      var sizeParams = me.getHealthCheckSummarySizeParams(healthcheckdata);
                      me.healthCheckWindow.setSize(sizeParams.width, sizeParams.height + (bbarItems.length > 0 ? me.rhcToolbarHeight : 0));
                      if (x && y) {
                          me.healthCheckWindow.setPosition(x, y);
                      }

                      var bbar = me.healthCheckWindow.getBottomToolbar();

                      if (bbar) {
                          Sonatype.Events.fireEvent('healthCheckWindowInit', me, bbarItems, record, healthcheckdata);

                          bbar.removeAll();
                          if (bbarItems.length > 0) {
                              for ( var i = 0; i < bbarItems.length; i++) {
                                  bbar.add(bbarItems[i]);
                              }
                          }
                      }
                  }
              }
          }
      };

      me.closeHealthCheckWindow = function() {
          if (me.healthCheckWindow != null) {
              me.healthCheckWindow.close();
              me.healthCheckWindow = null;
          }
      };

      me.updateHealthCheck = function(record, enabled) {
          Ext.Ajax.request({
              method : 'PUT',
              jsonData : {
                  id : record.get('id'),
                  status : enabled ? 'enabled' : 'disabled'
              },
              url : Sonatype.HealthCheckUtil.getBase() + '/healthCheckStatus',
              callback : function(options, success, response) {
                  // refresh the column data
                  if (success) {
                      var healthCheckData = me.healthCheckDataMap[record.get('id')];
                      if (healthCheckData) {
                          if (enabled) {
                              healthCheckData.status = 'analyzing';
                              me.rhcAnalyzingState = true;
                          } else {
                              healthCheckData.status = 'disabled';
                              if (me.healthCheckWindow != null && me.healthCheckWindow.healthcheckdata.id == healthCheckData.id) {
                                  me.closeHealthCheckWindow();
                              }
                          }

                          me.healthCheckDataMap[record.get('id')] = healthCheckData;
                          me.grid.getView().refreshRow(record);
                      }
                  }
              }
          });
      };

      me.generateHealthCheckTooltip = function() {
          if (me.healthCheckTooltip) {
              me.healthCheckTooltip.destroy();
          }

          me.healthCheckTooltip = new Ext.ToolTip({
              target : me.grid.getView().mainBody,
              delegate : '.x-grid3-cell',
              maxWidth : 500,
              mouseOffset : [ 0, 0 ],
              renderTo : document.body,
              hideDelay : 5000,
              baseCls : 'x-tipcustom',
              cls : 'healthCheckTooltip',
              items : [ {
                  xtype : 'panel',
                  frame : true,
                  html : ''
              } ],
            listeners: {
              beforeshow: function(tip) {
                var rowIndex = me.grid.getView().findRowIndex(tip.triggerElement);
                var cellIndex = me.grid.getView().findCellIndex(tip.triggerElement);
                var record = me.grid.getStore().getAt(rowIndex);

                if (!record || Sonatype.MessageBox.isVisible()) {
                  return false;
                }

                if (cellIndex == me.grid.getColumnModel().getIndexById('healthCheckColumn')) {
                  var healthCheckData = me.healthCheckDataMap[record.get('id')];

                  if (me.healthCheckStatusError) {
                    tip.items.get(0).body.update('<span><h2>Repository Health Check Status Error</h2>' +
                      'Nexus was unable to successfully retrieve the Repository Health Check status of the repositories.');
                    return;
                  }
                  else if (healthCheckData && healthCheckData.status == 'analyzing') {
                    tip.items.get(0).body.update('<span><h2>The Analysis is Under Way</h2>The contents of your repository are being analyzed.  ' +
                      'This process should only take a few minutes.<br><br>When complete, the ANALYZING button will be ' +
                      'replaced by a set of icons that indicate how many security and licensing issues were discovered.' +
                      '<br><br>Hover your mouse over these icons to see additional information about the issues that were found.</span>');
                    return;
                  }
                  else {
                    //no tooltip for this entry
                    tip.disable();
                  }
                }
                return false;
              }
            }
          });
      };

      Sonatype.Events.addListener('repositoryPanelInit', function(repoPanel) {
          if (Sonatype.HealthCheckUtil.hasReadPerm()) {
              me.reset({
                  grid : repoPanel.gridPanel
              });
          }
          return true;
      });

      Sonatype.Events.addListener('repositoryGridInit', function(repoPanel, columns, toolbar) {
          if (Sonatype.HealthCheckUtil.hasReadPerm()) {
              me.updateColumns(columns);
          }
          return true;
      });
  };

  Ext.onReady(function() {
      Sonatype.HealthCheckRepositoryContrib = new HealthCheckRepositoryContrib();
  });
});
